<?php
/**
 * 2018-2022 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2022 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

class MxbAdvdelslipUtils
{
    public static function getConfigValue($key, $default = null, $type = null, $save = false)
    {
        /*
        // dedicated management for following keys:
        if ($key == 'MXB_TOOLS_GLOBAL_CUSTOM_CSS' && !Tools::getIsset($key)) {
            $content = MxbDocnumUtils::readCssFile('customcss/global.css', 'css/_global_default.css');
            return $content;
        }
        */
        if (Tools::getIsset($key)) {
            if ($save) {
                self::setConfigValue($key, Tools::getValue($key), $type);
            }
            $ret = self::setValueType(Tools::getValue($key), $type);
            return $ret;
        }
        $ret = Configuration::getGlobalValue($key);
        if ($default !== null && $ret === false) {
            $ret = $default;
        }
        if ($save && $default !== null) {
            self::setConfigValue($key, $ret, $type);
        }
        return self::setValueType($ret, $type);
    }

    public static function setConfigValue($key, $value, $type = null)
    {
        Configuration::updateGlobalValue($key, self::setValueType($value, $type));
    }

    public static function setValueType($value, $type = null)
    {
        if ($type == 'string') {
            return $value;
        } elseif ($type == 'int') {
            return (int) $value;
        } elseif ($type == 'bool') {
            return (int)$value;
        }
        return $value;
    }

    /**
     * Get values for the inputs.
     */
    public static function getConfigFormValues($type = 'all')
    {
        $fields_values = array();
        if ($type == 'all' || $type == 'config') {
            foreach (Mxb_Advdelslip::configValues() as $value) {
                if ($value['lang']) {
                    foreach (Language::getLanguages(false) as $lang) {
                        $fields_values[$value['name']][$lang['id_lang']] = Tools::getValue(
                            $value['name'] . (int)$lang['id_lang'],
                            Configuration::getGlobalValue($value['name'], (int)$lang['id_lang'])
                        );
                    }
                } else {
                    $fields_values[$value['name']] = self::getConfigValue($value['name']);
                }
            }
        }
        if ($type == 'all' || $type == 'tools') {
            foreach (Mxb_Advdelslip::toolsValues() as $value) {
                if ($value['lang']) {
                    foreach (Language::getLanguages(false) as $lang) {
                        $fields_values[$value['name']][$lang['id_lang']] = Tools::getValue(
                            $value['name'] . (int)$lang['id_lang'],
                            Configuration::getGlobalValue($value['name'], (int)$lang['id_lang'])
                        );
                    }
                } else {
                    $fields_values[$value['name']] = self::getConfigValue($value['name']);
                }
            }
        }
        return $fields_values;
    }

    public static function isModuleEnabled($moduleName)
    {
        return Module::isEnabled($moduleName);
    }


    public static function getCommonAdminTplValues($context = null)
    {
        if ($context === null) {
            $context = Context::getContext();
        }
        return array('languages' => Language::getLanguages(false),
            'defaultFormLanguage' => (int) $context->language->id,
            'module_dir' => self::getModulePath(),
            'admin_template_dir' => self::getModulePath() . '/views/templates/admin',
            'admin_menu_tpl' => self::getModulePath() . '/views/templates/admin/_partials/menu.tpl',
            'admin_token' => Tools::getAdminTokenLite('AdminModules'),
            'mxb_module_base_url' => self::getModulePathUri(),
            'uri' => self::getModulePathUri(), // duplicated: TODO: ...
            // ... search where "uri" was used and substitute it with "mxb_module_base_url"
        );
    }

    public static function getModulePathUri()
    {
        //$uri =  Tools::getHttpHost(true).__PS_BASE_URI__ . '/modules/mxb_tools/'; // first method: OK
        $uri = self::path2url(self::getModulePath()); // second method: OK
        if (Tools::substr($uri, -1) != DIRECTORY_SEPARATOR) {
            $uri .= DIRECTORY_SEPARATOR;
        }
        return $uri;
    }

    public static function getModulePath($module = 'mxb_advdelslip')
    {
        return _PS_MODULE_DIR_ . $module . DIRECTORY_SEPARATOR;
    }

    public static function path2url($file, $Protocol = null)
    {
        if ($Protocol === null) {
            $Protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . '://';
        }
        $DOCUMENT_ROOT = $_SERVER['DOCUMENT_ROOT'];
        if (Tools::substr($DOCUMENT_ROOT, -1) != DIRECTORY_SEPARATOR) {
            $DOCUMENT_ROOT .= DIRECTORY_SEPARATOR;
        }
        $HTTP_HOST = $_SERVER['HTTP_HOST'];
        if (Tools::substr($HTTP_HOST, -1) != DIRECTORY_SEPARATOR) {
            $HTTP_HOST .= DIRECTORY_SEPARATOR;
        }
        return $Protocol.$HTTP_HOST.str_replace($DOCUMENT_ROOT, '', $file);
    }

    public static function getServerHttp($get_uri = true)
    {
        return (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http")
            . "://$_SERVER[HTTP_HOST]" . ($get_uri ? "$_SERVER[REQUEST_URI]" : "/");
    }

    public static function getTab($controller, $module = 'mxb_advdelslip')
    {
        if (class_exists('PrestaShopCollection')) {
            $collection = new PrestaShopCollection('Tab');
        } else {
            $collection = new Collection('Tab');
        }
        $collection->where('class_name', '=', $controller);
        if ($module) {
            $collection->where('module', '=', $module);
        }
        $tab = $collection->getFirst();
        if (!$tab) {
            $tab = new Tab();
        }
        $tab->class_name = $controller;
        $tab->module = $module;
        return $tab;
    }
}
