<?php
/**
 * 2018-2022 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2022 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

require_once _PS_MODULE_DIR_.'mxb_advdelslip/classes/MxbAdvdelslipUtils.php';
require_once _PS_MODULE_DIR_.'mxb_advdelslip/classes/MxbAdvdelslipDeliverySlipData.php';

/*
if ( file_exists(_PS_MODULE_DIR_.'mxb_tools/classes/MxbToolsDebug.php') ) {
    require_once _PS_MODULE_DIR_.'mxb_tools/classes/MxbToolsDebug.php';
}
*/
if (!defined('_PS_VERSION_')) {
    exit;
}

class Mxb_Advdelslip extends Module
{
    //const MODULE_NAME = 'mxb_advdelslip';
    const ROOT_CONTROLLER = 'AdminMxbAdvdelslip';
    // the following version variable is for development, used do update databases and other stuff without reinstalling the module
    // please check checkForUpdates() function
    // use int value !
    const MXB_DEV_VERSION = 1;

    protected $config_form = false;
    //protected $tabs = array();

    public $cache_update_additional_cost = false;
    public $cache_saving_dress_size = false;


    // TRANSLATIONS
    public static $l = array();

    // use static array for translations from static classes where is:
    //        $module = Module::getInstanceByName('mxb_tools');
    //        $module->l(string)
    public function isUsingNewTranslationSystem() {
        return false; // force using old translation system: wait for 1.7.8 to use new translation, when module translations will be exportable.
    }

    public function initTranslation()
    {
       if (self::isInstalled($this->name)) {
          self::$l = array(
                // sample element
             'MXB Documents Numbering' => $this->l('MXB Documents Numbering'),
          );
       }
    }



    public static function configValues() {
        return array(
            array('name' => 'MXB_ADVDELSLIP_VERSION', 'type' => 'int', 'default' => 1, 'lang' => false),
            array('name' => 'MXB_ADVDELSLIP_ADDITIONAL_DATA_ENABLED', 'type' => 'bool', 'default' => true, 'lang' => false),
        );
    }

    // not used
    public static function toolsValues() {
        return array();
    }

    public function __construct()
    {
        $this->name = 'mxb_advdelslip';
        $this->rootController = self::ROOT_CONTROLLER;
        $this->tab = 'administration'; //'front_office_features';
        $this->dev_version = self::MXB_DEV_VERSION;
        $this->version = '1.0.0';
        $this->author = 'MXB Informatica';
        $this->need_instance = 1;
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
        $this->bootstrap = true;     /* Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6) */

        parent::__construct();

        $this->displayName = $this->l('MXB Advanced delivery slip');
        $this->description = $this->l('Adds details table to delivery slips');

        $this->checkForUpdates();

        // check configuration values and set the not existing ones
        $this->setModuleValues();
        // DEBUG: remove for production!
         //$this->createAdminTabs();
        //$this->registerHook('displayAdminOrderContentShip'); // hook for order page, 1.7.
        //$this->registerHook('displayAdminOrderTabContent'); // hook for order page, 1.7.7
        //$this->registerHook('actionAdminControllerSetMedia'); // hook for order page, 1.7.7

    }




    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     *
     * LIST OF HOOKS PS1.7
     * http://build.prestashop.com/news/new-updated-hooks-1-7-1-0/
     */
    public function install()
    {
        include(dirname(__FILE__).'/sql/install.php');

        $res = parent::install();

        $this->setModuleValues();
        $this->installModuleFiles();

        $res &= $this->registerHook('header');
        $res &= $this->registerHook('backOfficeHeader');
        //$res &= $this->registerHook('actionInvoiceNumberFormatted');
        //$res &= $this->registerHook('actionValidateOrder');
        $res &= $this->registerHook('displayPDFDeliverySlip');
        $res &= $this->registerHook('displayAdminOrderContentShip'); // hook for order page, 1.7.6
        $res &= $this->registerHook('displayAdminOrderTabContent'); // hook for order page, 1.7.7
        $res &= $this->registerHook('actionAdminControllerSetMedia');

        return (bool) $res;

    }

    public function uninstall()
    {
        $this->unsetModuleValues();
        include(dirname(__FILE__).'/sql/uninstall.php');
        return parent::uninstall();
    }

    protected function setModuleValues() {

        foreach (Mxb_Advdelslip::configValues() as $value) {
            if ((bool)Configuration::hasKey($value['name']) === false) {
                Configuration::updateGlobalValue($value['name'], $this->setValueType($value['default'], $value['type']) );
            }
        }
        foreach (Mxb_Advdelslip::toolsValues() as $value) {
            if ((bool)Configuration::hasKey($value['name']) === false)
                Configuration::updateGlobalValue($value['name'], $this->setValueType($value['default'], $value['type']));
        }
    }

    protected function unsetModuleValues() {
        foreach (Mxb_Advdelslip::configValues() as $value) {
            Configuration::deleteByName($value['name']);
        }
        foreach (Mxb_Advdelslip::toolsValues() as $value) {
            Configuration::deleteByName($value['name']);
        }
    }

    protected function setValueType($value, $type = null) {
        return MxbAdvdelslipUtils::setValueType($value, $type);
    }

    protected function installModuleFiles() {
        return true;
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        return Tools::redirectAdmin($this->context->link->getAdminLink($this->rootController . 'Configuration'));
    }

    /**
    * Add the CSS & JavaScript files you want to be loaded in the BO.
    */
    public function hookBackOfficeHeader()
    {
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
    }

    /**
     * Located in: /classes/order/OrderInvoice.php
     */

    /*
    public function hookActionPDFInvoiceRender($hookParams)
    {
           //Mxb_ToolsDebug::WriteLog($this->name, 'hookActionPDFInvoiceRender()',  'hookParams: ' . print_r($hookParams, true));
    }
    */

    public function hookDisplayPDFDeliverySlip($hookParams)
    {
        //MxbToolsDebug::WriteLog($this->name, 'hookDisplayPDFDeliverySlip()',  'hookParams: ' . print_r($hookParams, true), true);
        $id_order = (isset($hookParams['object']->id_order) ? $hookParams['object']->id_order : 0);
        $deliverySlipData = MxbAdvdelslipDeliverySlipData::getDeliverySlipDataForSmarty($id_order);
        $deliverySlipData['disclaimer'] = $this->l('Delivery slip footer disclaimer');
        $this->context->smarty->assign('deliverySlipData', $deliverySlipData);
        
        return $this->context->smarty->fetch(_PS_MODULE_DIR_.'mxb_advdelslip/views/templates/hook/pdf/delivery-slip.style-tab.tpl')
            . $this->context->smarty->fetch(_PS_MODULE_DIR_.'mxb_advdelslip/views/templates/hook/pdf/delivery-slip.adv-table-tab.tpl');
    }

    /* 1.7.6: admin order page: add delivery slip data */
    public function hookDisplayAdminOrderContentShip($hookParams)
    {
        //MxbToolsDebug::writeLog($this->name, 'hookDisplayAdminOrderContentShip()',  'hookParams = ' . print_r($hookParams, true), true);
        
        // if mxbmemberprogram or gadgetcard module are installed, let it manage these values
        if ((bool)Configuration::getGlobalValue('MXB_ADVDELSLIP_ADDITIONAL_DATA_ENABLED') === false || MxbAdvdelslipUtils::isModuleEnabled('mxbmemberprogram') || MxbAdvdelslipUtils::isModuleEnabled('mxb_docnum') || MxbAdvdelslipUtils::isModuleEnabled('gadgetcard')) {
            return;
        }
        $id_order = 0;
        if (isset($hookParams['order'])) {
            $id_order = (int) $hookParams['order']->id;
        } elseif (isset($hookParams['id_order'])) {
            $id_order = (int) $hookParams['id_order'];
        }
        $deliverySlipData = MxbAdvdelslipDeliverySlipData::getDeliverySlipDataForSmarty($id_order);
        $deliverySlipData['total_shipping_tax_excl'] = $hookParams['order']->total_shipping_tax_excl;
        $this->context->smarty->assign('mxbDeliverySlipData', $deliverySlipData);
        return $this->display(__FILE__, 'admin/mxb-order-delivery-slip-data.tpl');
    }

    /* 1.7.7: admin order page: add delivery slip data */
    public function hookDisplayAdminOrderTabContent($hookParams) {
        return $this->hookDisplayAdminOrderContentShip($hookParams);
    }

    public function hookActionAdminControllerSetMedia()
    {
        //$this->context->controller->addJS($this->_path.'views/js/mxb_tools.js');
        $this->context->controller->addJS($this->_path.'views/js/back.js');
        $this->context->controller->addCSS($this->_path.'views/css/back.css');
    }

    protected function checkForUpdates() {
        // check for back-office
        //if (!Tools::getIsset('controller'))
        //    return false;
        $version = Configuration::getGlobalValue('MXB_ADVDELSLIP_VERSION');
        if ($version === 0) {
            // module was fresh of installation: no updates are needed: update current version and exit
            Configuration::updateGlobalValue('MXB_ADVDELSLIP_VERSION', self::MXB_DEV_VERSION);
            return false;
        }
        if ( (int)self::MXB_DEV_VERSION <= (int)$version ) {
            return false;
        }

        $confirmations = array();
        $errors = array();


        for ($i = $version+1; $i <= self::MXB_DEV_VERSION; $i++) {

            $version = $i;
            switch ($version) {
                case 1:
                    break;

                case 2:
                    break;
            }

        }
        // update current version
        Configuration::updateGlobalValue('MXB_ADVDELSLIP_VERSION', $version);
        return true;
    }
}
